package com.isu.gaswellwatch.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.isu.gaswellwatch.config.SnowflakeConfig;
import com.isu.gaswellwatch.dao.DepartmentDao;
import com.isu.gaswellwatch.dao.DeviceDao;
import com.isu.gaswellwatch.dto.DepartmentDTO;
import com.isu.gaswellwatch.dto.DepartmentEditDTO;
import com.isu.gaswellwatch.dto.DeviceCreateDTO;
import com.isu.gaswellwatch.dto.DeviceEditDTO;
import com.isu.gaswellwatch.entity.BlockDepartment;
import com.isu.gaswellwatch.entity.Department;
import com.isu.gaswellwatch.entity.Device;
import com.isu.gaswellwatch.exception.BusinessException;
import com.isu.gaswellwatch.service.BlockDepartmentService;
import com.isu.gaswellwatch.service.DepartmentService;
import com.isu.gaswellwatch.service.DeviceService;
import com.isu.gaswellwatch.service.GasWellService;
import com.isu.gaswellwatch.utils.ConverterUtil;
import com.isu.gaswellwatch.vo.DepartmentVO;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service("deviceService")
@Transactional(rollbackFor = Exception.class)
public class DeviceServiceImpl extends ServiceImpl<DeviceDao, Device> implements DeviceService {

    @Resource
    private SnowflakeConfig snowflakeConfig;

    @Resource
    private DeviceDao deviceDao;
    @Resource
    private GasWellService gasWellService;

    @Override
    public Page<Device> page(Integer currentPage, Integer pageSize, String gasWellName, String gasStationName,Long deviceTypeId){
        Page<Device> page = deviceDao.page(new Page<>(currentPage, pageSize),gasWellName,gasStationName,deviceTypeId);

        //TODO 从Redis获取设备数据

        return ConverterUtil.convertPage(page, Device.class);
    }

    @Override
    public void add(DeviceCreateDTO deviceCreateDTO){
        //查重
        List<Device> list = list(new LambdaQueryWrapper<Device>().eq(Device::getId, deviceCreateDTO.getCode()));
        if(CollectionUtil.isNotEmpty(list)) {
            throw new BusinessException("已有相同设备编码，请重新输入");
        }
        Device device = ConverterUtil.convert(deviceCreateDTO, Device.class);
        //code必须为整形，用于在缓存中查询设备的上报数据
        device.setId(Long.valueOf(deviceCreateDTO.getCode()));
        save(device);

        //在气井中绑定设备
        gasWellService.bindDevice(deviceCreateDTO.getGasWell(),device.getId());
    }

    @Override
    public void edit(DeviceEditDTO deviceEditDTO){
        updateById(ConverterUtil.convert(deviceEditDTO, Device.class));
    }

    @Override
    public void delete(Long id){
        //删除设备
            removeById(id);
            //解绑气井
            gasWellService.unbindDevice(id);
    }

    @Override
    public Device getDevice(Long id) {
        return null;
    }

    @Override
    public void getDeviceControlData(Long id) {

    }

    @Override
    public void getDeviceHistoryData(Long id) {

    }

    @Override
    public void getDeviceLogData(Long id) {

    }


}

